package com.sun.java.swing.plaf.windows;

import java.awt.event.MouseEvent;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Graphics;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.JSlider;
import javax.swing.plaf.basic.BasicSliderUI;

public class WindowsSliderUI extends BasicSliderUI
{
    private boolean rollover;
    private boolean pressed;
    
    public WindowsSliderUI(JSlider b) {
        super(b);
        this.rollover = false;
        this.pressed = false;
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new WindowsSliderUI((JSlider)component);
    }
    
    @Override
    protected BasicSliderUI.TrackListener createTrackListener(JSlider slider) {
        return new WindowsSliderUI.WindowsTrackListener();
    }
    
    @Override
    public void paintTrack(Graphics g) {
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            int n = this.slider.getOrientation() == 1;
            XPStyle.Skin skin = xp.getSkin(this.slider, n ? TMSchema.Part.TKP_TRACKVERT : TMSchema.Part.TKP_TRACK);
            if (!(n == 0)) {
                skin.paintSkin(g, this.trackRect.x + (this.trackRect.width - skin.getWidth()) / 2, this.trackRect.y, skin.getWidth(), this.trackRect.height, null);
            }
            else {
                skin.paintSkin(g, this.trackRect.x, this.trackRect.y + (this.trackRect.height - skin.getHeight()) / 2, this.trackRect.width, skin.getHeight(), null);
            }
        }
        else {
            super.paintTrack(g);
        }
    }
    
    @Override
    protected void paintMinorTickForHorizSlider(Graphics g, Rectangle tickBounds, int x) {
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            g.setColor(xp.getColor(this.slider, TMSchema.Part.TKP_TICS, null, TMSchema.Prop.COLOR, Color.black));
        }
        super.paintMinorTickForHorizSlider(g, tickBounds, x);
    }
    
    @Override
    protected void paintMajorTickForHorizSlider(Graphics g, Rectangle tickBounds, int x) {
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            g.setColor(xp.getColor(this.slider, TMSchema.Part.TKP_TICS, null, TMSchema.Prop.COLOR, Color.black));
        }
        super.paintMajorTickForHorizSlider(g, tickBounds, x);
    }
    
    @Override
    protected void paintMinorTickForVertSlider(Graphics g, Rectangle tickBounds, int y) {
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            g.setColor(xp.getColor(this.slider, TMSchema.Part.TKP_TICSVERT, null, TMSchema.Prop.COLOR, Color.black));
        }
        super.paintMinorTickForVertSlider(g, tickBounds, y);
    }
    
    @Override
    protected void paintMajorTickForVertSlider(Graphics g, Rectangle tickBounds, int y) {
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            g.setColor(xp.getColor(this.slider, TMSchema.Part.TKP_TICSVERT, null, TMSchema.Prop.COLOR, Color.black));
        }
        super.paintMajorTickForVertSlider(g, tickBounds, y);
    }
    
    @Override
    public void paintThumb(Graphics g) {
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            TMSchema.Part xpThumbPart = this.getXPThumbPart();
            TMSchema.State state = TMSchema.State.NORMAL;
            if (this.slider.hasFocus()) {
                state = TMSchema.State.FOCUSED;
            }
            if (this.rollover) {
                state = TMSchema.State.HOT;
            }
            if (this.pressed) {
                state = TMSchema.State.PRESSED;
            }
            if (!this.slider.isEnabled()) {
                state = TMSchema.State.DISABLED;
            }
            xp.getSkin(this.slider, xpThumbPart).paintSkin(g, this.thumbRect.x, this.thumbRect.y, state);
        }
        else {
            super.paintThumb(g);
        }
    }
    
    @Override
    protected Dimension getThumbSize() {
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            Dimension dimension = new Dimension();
            XPStyle.Skin skin = xp.getSkin(this.slider, this.getXPThumbPart());
            dimension.width = skin.getWidth();
            dimension.height = skin.getHeight();
            return dimension;
        }
        return super.getThumbSize();
    }
    
    private TMSchema.Part getXPThumbPart() {
        int n = this.slider.getOrientation() == 1;
        int leftToRight = this.slider.getComponentOrientation().isLeftToRight();
        Boolean b = (Boolean)this.slider.getClientProperty("Slider.paintThumbArrowShape");
        if ((!this.slider.getPaintTicks() && b == null) || b == Boolean.FALSE) {
            return n ? TMSchema.Part.TKP_THUMBVERT : TMSchema.Part.TKP_THUMB;
        }
        return n ? (leftToRight ? TMSchema.Part.TKP_THUMBRIGHT : TMSchema.Part.TKP_THUMBLEFT) : TMSchema.Part.TKP_THUMBBOTTOM;
    }
    
    private class WindowsTrackListener extends BasicSliderUI.TrackListener
    {
        @Override
        public void mouseMoved(MouseEvent e) {
            this.updateRollover(WindowsSliderUI.this.thumbRect.contains(e.getX(), e.getY()));
            super.mouseMoved(e);
        }
        
        @Override
        public void mouseEntered(MouseEvent e) {
            this.updateRollover(WindowsSliderUI.this.thumbRect.contains(e.getX(), e.getY()));
            super.mouseEntered(e);
        }
        
        @Override
        public void mouseExited(MouseEvent e) {
            this.updateRollover(false);
            super.mouseExited(e);
        }
        
        @Override
        public void mousePressed(MouseEvent e) {
            this.updatePressed(WindowsSliderUI.this.thumbRect.contains(e.getX(), e.getY()));
            super.mousePressed(e);
        }
        
        @Override
        public void mouseReleased(MouseEvent e) {
            this.updatePressed(false);
            super.mouseReleased(e);
        }
        
        public void updatePressed(boolean b) {
            if (!WindowsSliderUI.this.slider.isEnabled()) {
                return;
            }
            if (WindowsSliderUI.this.pressed != b) {
                WindowsSliderUI.this.pressed = b;
                WindowsSliderUI.this.slider.repaint(WindowsSliderUI.this.thumbRect);
            }
            return;
        }
        
        public void updateRollover(boolean b) {
            if (!WindowsSliderUI.this.slider.isEnabled()) {
                return;
            }
            if (WindowsSliderUI.this.rollover != b) {
                WindowsSliderUI.this.rollover = b;
                WindowsSliderUI.this.slider.repaint(WindowsSliderUI.this.thumbRect);
            }
            return;
        }
    }
}
